@@ -29,7 +29,7 @@
 
 from __future__ import division
 
-__version__ = '0.6'
+__version__ = '0.7'
 __all__ = ['Flavor', 'Table', 'Values', 'Literal', 'Column', 'Join',
     'Asc', 'Desc']
 
@@ -72,8 +72,10 @@ class Flavor(object):
         function_mapping - dictionary with Function to replace
     '''
 
-    def __init__(self, max_limit=None, paramstyle='format', ilike=False,
-            function_mapping=None):
+    def __init__(self, limitstyle='limit', max_limit=None, paramstyle='format',
+            ilike=False, function_mapping=None):
+        assert limitstyle in ['fetch', 'limit']
+        self.limitstyle = limitstyle
         self.max_limit = max_limit
         self.paramstyle = paramstyle
         self.ilike = ilike
@@ -241,6 +243,25 @@ class FromItem(object):
     def join(self, right, type_='INNER', condition=None):
         return Join(self, right, type_=type_, condition=condition)
 
+    def lateral(self):
+        return Lateral(self)
+
+
+class Lateral(FromItem):
+    __slots__ = ('_from_item',)
+
+    def __init__(self, from_item):
+        self._from_item = from_item
+
+    def __str__(self):
+        template = '%s'
+        if isinstance(self._from_item, Query):
+            template = '(%s)'
+        return 'LATERAL ' + template % self._from_item
+
+    def __getattr__(self, name):
+        return getattr(self._from_item, name)
+
 
 class With(FromItem):
     __slots__ = ('columns', 'query', 'recursive')
@@ -308,17 +329,6 @@ class SelectQuery(WithQuery):
             assert isinstance(value, (int, long))
         self._limit = value
 
-    @property
-    def _limit_str(self):
-        limit = ''
-        if self.limit is not None:
-            limit = ' LIMIT %s' % self.limit
-        elif self.offset:
-            max_limit = Flavor.get().max_limit
-            if max_limit:
-                limit = ' LIMIT %s' % max_limit
-        return limit
-
     @property
     def offset(self):
         return self._offset
@@ -330,11 +340,27 @@ class SelectQuery(WithQuery):
         self._offset = value
 
     @property
-    def _offset_str(self):
-        offset = ''
-        if self.offset:
-            offset = ' OFFSET %s' % self.offset
-        return offset
+    def _limit_offset_str(self):
+        if Flavor.get().limitstyle == 'limit':
+            offset = ''
+            if self.offset:
+                offset = ' OFFSET %s' % self.offset
+            limit = ''
+            if self.limit is not None:
+                limit = ' LIMIT %s' % self.limit
+            elif self.offset:
+                max_limit = Flavor.get().max_limit
+                if max_limit:
+                    limit = ' LIMIT %s' % max_limit
+            return limit + offset
+        else:
+            offset = ''
+            if self.offset:
+                offset = ' OFFSET (%s) ROWS' % self.offset
+            fetch = ''
+            if self.limit is not None:
+                fetch = ' FETCH FIRST (%s) ROWS ONLY' % self.limit
+            return offset + fetch
 
 
 class Select(FromItem, SelectQuery):
@@ -419,6 +445,23 @@ class Select(FromItem, SelectQuery):
         else:
             return str(column)
 
+    def _window_functions(self):
+        from sql.functions import WindowFunction
+        from sql.aggregate import Aggregate
+        windows = set()
+        for column in self.columns:
+            window_function = None
+            if isinstance(column, (WindowFunction, Aggregate)):
+                window_function = column
+            elif (isinstance(column, As)
+                    and isinstance(column.expression,
+                        (WindowFunction, Aggregate))):
+                window_function = column.expression
+            if (window_function and window_function.window
+                    and window_function.window not in windows):
+                windows.add(window_function.window)
+                yield window_function
+
     def __str__(self):
         with AliasManager():
             from_ = str(self.from_)
@@ -435,13 +478,18 @@ class Select(FromItem, SelectQuery):
             having = ''
             if self.having:
                 having = ' HAVING ' + str(self.having)
+            window = ''
+            windows = [f.window for f in self._window_functions()]
+            if windows:
+                window = ' WINDOW ' + ', '.join(
+                    '"%s" AS (%s)' % (w.alias, w) for w in windows)
             for_ = ''
             if self.for_ is not None:
                 for_ = ' ' + ' '.join(map(str, self.for_))
             return (self._with_str()
                 + 'SELECT %s FROM %s' % (columns, from_)
-                + where + group_by + having + self._order_by_str
-                + self._limit_str + self._offset_str + for_)
+                + where + group_by + having + window + self._order_by_str
+                + self._limit_offset_str + for_)
 
     @property
     def params(self):
@@ -457,11 +505,13 @@ class Select(FromItem, SelectQuery):
         if self.group_by:
             for expression in self.group_by:
                 p.extend(expression.params)
-        if self.having:
-            p.extend(self.having.params)
         if self.order_by:
             for expression in self.order_by:
                 p.extend(expression.params)
+        if self.having:
+            p.extend(self.having.params)
+        for window_function in self._window_functions():
+            p.extend(window_function.window.params)
         return tuple(p)
 
 
@@ -724,7 +774,7 @@ class CombiningQuery(FromItem, SelectQuery):
         with AliasManager():
             operator = ' %s %s' % (self._operator, 'ALL ' if self.all_ else '')
             return (operator.join(map(str, self.queries)) + self._order_by_str
-                + self._limit_str + self._offset_str)
+                + self._limit_offset_str)
 
     @property
     def params(self):
@@ -1166,6 +1216,115 @@ class Cast(Expression):
             return (self.expression,)
 
 
+class Window(object):
+    __slots__ = ('_partition', '_order_by', '_frame', '_start', '_end')
+
+    def __init__(self, partition, order_by=None,
+            frame=None, start=None, end=0):
+        super(Window, self).__init__()
+        self._partition = None
+        self._order_by = None
+        self._frame = None
+        self._start = None
+        self._end = None
+        self.partition = partition
+        self.order_by = order_by
+        self.frame = frame
+        self.start = start
+        self.end = end
+
+    @property
+    def partition(self):
+        return self._partition
+
+    @partition.setter
+    def partition(self, value):
+        assert all(isinstance(e, Expression) for e in value)
+        self._partition = value
+
+    @property
+    def order_by(self):
+        return self._order_by
+
+    @order_by.setter
+    def order_by(self, value):
+        if value is not None:
+            if isinstance(value, Expression):
+                value = [value]
+            assert all(isinstance(col, Expression) for col in value)
+        self._order_by = value
+
+    @property
+    def frame(self):
+        return self._frame
+
+    @frame.setter
+    def frame(self, value):
+        if value:
+            assert value in ['RANGE', 'ROWS']
+        self._frame = value
+
+    @property
+    def start(self):
+        return self._start
+
+    @start.setter
+    def start(self, value):
+        if value:
+            assert isinstance(value, (int, long))
+        self._start = value
+
+    @property
+    def end(self):
+        return self._end
+
+    @end.setter
+    def end(self, value):
+        if value:
+            assert isinstance(value, (int, long))
+        self._end = value
+
+    @property
+    def alias(self):
+        return AliasManager.get(self)
+
+    def __str__(self):
+        partition = ''
+        if self.partition:
+            partition = 'PARTITION BY ' + ', '.join(map(str, self.partition))
+        order_by = ''
+        if self.order_by:
+            order_by = ' ORDER BY ' + ', '.join(map(str, self.order_by))
+
+        def format(frame, direction):
+            if frame is None:
+                return 'UNBOUNDED %s' % direction
+            elif not frame:
+                return 'CURRENT ROW'
+            elif frame < 0:
+                return '%s PRECEDING' % -frame
+            elif frame > 0:
+                return '%s FOLLOWING' % frame
+
+        frame = ''
+        if self.frame:
+            start = format(self.start, 'PRECEDING')
+            end = format(self.end, 'FOLLOWING')
+            frame = ' %s BETWEEN %s AND %s' % (self.frame, start, end)
+        return partition + order_by + frame
+
+    @property
+    def params(self):
+        p = []
+        if self.partition:
+            for expression in self.partition:
+                p.extend(expression.params)
+        if self.order_by:
+            for expression in self.order_by:
+                p.extend(expression.params)
+        return tuple(p)
+
+
 class Order(Expression):
     __slots__ = ('expression')
     _sql = ''
