@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (c) 2011-2013, Cédric Krier
+# Copyright (c) 2011-2015, Cédric Krier
 # Copyright (c) 2013-2014, Nicolas Évrard
-# Copyright (c) 2011-2013, B2CK
+# Copyright (c) 2011-2015, B2CK
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -31,8 +31,8 @@ import unittest
 import warnings
 from copy import deepcopy
 
-from sql import Table, Join, Union, Literal, Flavor, For, With
-from sql.functions import Now, Function
+from sql import Table, Join, Union, Literal, Flavor, For, With, Window, Select
+from sql.functions import Now, Function, Rank, DatePart
 from sql.aggregate import Min
 
 
@@ -195,24 +195,25 @@ class TestSelect(unittest.TestCase):
         self.assertEqual(query.params, (1,))
 
     def test_select_limit_offset(self):
-        query = self.table.select(limit=50, offset=10)
-        self.assertEqual(str(query),
-            'SELECT * FROM "t" AS "a" LIMIT 50 OFFSET 10')
-        self.assertEqual(query.params, ())
-
-        query.limit = None
-        self.assertEqual(str(query),
-            'SELECT * FROM "t" AS "a" OFFSET 10')
-        self.assertEqual(query.params, ())
-
-        query.offset = 0
-        self.assertEqual(str(query),
-            'SELECT * FROM "t" AS "a"')
-        self.assertEqual(query.params, ())
-
-        flavor = Flavor(max_limit=-1)
-        Flavor.set(flavor)
         try:
+            Flavor.set(Flavor(limitstyle='limit'))
+            query = self.table.select(limit=50, offset=10)
+            self.assertEqual(str(query),
+                'SELECT * FROM "t" AS "a" LIMIT 50 OFFSET 10')
+            self.assertEqual(query.params, ())
+
+            query.limit = None
+            self.assertEqual(str(query),
+                'SELECT * FROM "t" AS "a" OFFSET 10')
+            self.assertEqual(query.params, ())
+
+            query.offset = 0
+            self.assertEqual(str(query),
+                'SELECT * FROM "t" AS "a"')
+            self.assertEqual(query.params, ())
+
+            Flavor.set(Flavor(limitstyle='limit', max_limit=-1))
+
             query.offset = None
             self.assertEqual(str(query),
                 'SELECT * FROM "t" AS "a"')
@@ -230,6 +231,27 @@ class TestSelect(unittest.TestCase):
         finally:
             Flavor.set(Flavor())
 
+    def test_select_offset_fetch(self):
+        try:
+            Flavor.set(Flavor(limitstyle='fetch'))
+            query = self.table.select(limit=50, offset=10)
+            self.assertEqual(str(query),
+                'SELECT * FROM "t" AS "a" '
+                'OFFSET (10) ROWS FETCH FIRST (50) ROWS ONLY')
+            self.assertEqual(query.params, ())
+
+            query.limit = None
+            self.assertEqual(str(query),
+                'SELECT * FROM "t" AS "a" OFFSET (10) ROWS')
+            self.assertEqual(query.params, ())
+
+            query.offset = 0
+            self.assertEqual(str(query),
+                'SELECT * FROM "t" AS "a"')
+            self.assertEqual(query.params, ())
+        finally:
+            Flavor.set(Flavor())
+
     def test_select_for(self):
         c = self.table.c
         query = self.table.select(c, for_=For('UPDATE'))
@@ -252,3 +274,49 @@ class TestSelect(unittest.TestCase):
             'WITH "a" AS (SELECT "b"."c1" FROM "t" AS "b") '
             'SELECT * FROM "a" AS "a"')
         self.assertEqual(query.params, ())
+
+    def test_window(self):
+        query = self.table.select(Min(self.table.c1,
+                window=Window([self.table.c2])))
+
+        self.assertEqual(str(query),
+            'SELECT MIN("a"."c1") OVER "b" FROM "t" AS "a" '
+            'WINDOW "b" AS (PARTITION BY "a"."c2")')
+        self.assertEqual(query.params, ())
+
+        query = self.table.select(Rank(window=Window([])))
+        self.assertEqual(str(query),
+            'SELECT RANK() OVER "b" FROM "t" AS "a" '
+            'WINDOW "b" AS ()')
+        self.assertEqual(query.params, ())
+
+        window = Window([self.table.c1])
+        query = self.table.select(
+            Rank(filter_=self.table.c1 > 0, window=window),
+            Min(self.table.c1, window=window))
+        self.assertEqual(str(query),
+            'SELECT RANK() FILTER (WHERE ("a"."c1" > %s)) OVER "b", '
+            'MIN("a"."c1") OVER "b" FROM "t" AS "a" '
+            'WINDOW "b" AS (PARTITION BY "a"."c1")')
+        self.assertEqual(query.params, (0,))
+
+        window = Window([DatePart('year', self.table.date_col)])
+        query = self.table.select(
+            Min(self.table.c1, window=window))
+        self.assertEqual(str(query),
+            'SELECT MIN("a"."c1") OVER "b" FROM "t" AS "a" '
+            'WINDOW "b" AS (PARTITION BY DATE_PART(%s, "a"."date_col"))')
+        self.assertEqual(query.params, ('year',))
+
+    def test_order_params(self):
+        with_ = With(query=self.table.select(self.table.c,
+                where=(self.table.c > 1)))
+        w = Window([Literal(8)])
+        query = Select([Literal(2), Min(self.table.c, window=w)],
+            from_=self.table.select(where=self.table.c > 3),
+            with_=with_,
+            where=self.table.c > 4,
+            group_by=[Literal(5)],
+            order_by=[Literal(6)],
+            having=Literal(7))
+        self.assertEqual(query.params, (1, 2, 3, 4, 5, 6, 7, 8))
